unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls;

type
  TTokenType = (tkComment, tkIdentifier, tkFunction, tkSymbol, tkNumber, tkSpace,
    tkString, tkUnknown, tkVar, tkUndefined);

  PHashInfo = ^THashInfo;

  THashInfo = record
    Key: ansistring;
    Kind: TTokenType;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    ComboBox1: TComboBox;
    Edit1: TEdit;
    ListBox1: TListBox;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
  private
    FHashList: array[0..255] of TList;
    procedure LoadHLTable(Path: string);
    procedure SaveHLTable(Path: string);
    { private declarations }
  public
    { public declarations }
  end;

var
  Form1: TForm1;

const CaseInsensitive = true;

implementation

{$R *.lfm}

procedure TForm1.FormCreate(Sender: TObject);
var
  i: integer;
begin
  for i := 0 to 255 do
    FHashList[i] := TList.Create;
end;

procedure TForm1.Button5Click(Sender: TObject);
var
  i, x: integer;
begin
  if OpenDialog1.Execute then
  begin
    LoadHLTable(OpenDialog1.FileName);
    ListBox1.Clear;
    for i := 0 to 255 do
      for x := 0 to FHashList[i].Count - 1 do
        ListBox1.Items.Add(PHashInfo(FHashList[i][x])^.Key);
  end;
end;

procedure TForm1.Button6Click(Sender: TObject);

var
  i, x: integer;
begin
  for x := 0 to 255 do
    for i := 0 to FHashList[x].Count - 1 do
      Dispose(PHashInfo(FHashList[x][i]));
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  x, i: integer;
begin
  for x := 0 to 255 do
    for i := 0 to FHashList[x].Count - 1 do
      if PHashInfo(FHashList[x][i])^.Key = ListBox1.Items[ListBox1.ItemIndex] then
      begin
        Dispose(PHashInfo(FHashList[x][i]));
        FHashList[x].Delete(i);
        ListBox1.Items.Delete(ListBox1.ItemIndex);
      end;
end;

procedure TForm1.Button3Click(Sender: TObject);

function GetHash(toHash: PChar; out Len: integer): byte;
begin
  Result := 0;
  Len := 0;
  while ToHash^ in ['_', '0'..'9', 'a'..'z', 'A'..'Z'] do
  begin
    if CaseInsensitive And (toHash^ in ['A'..'Z']) then
      Result := (Result + Ord(toHash^)-ord('A')+ord('a')) mod 256
    else
      Result := (Result + Ord(toHash^)) mod 256;
    Inc(ToHash);
    Inc(Len);
  end;
end;

var
  x: Byte;
  Dummy: Integer;
  tmp: PHashInfo;
begin
  x := GetHash(PChar(Edit1.Text), Dummy);
   ShowMessage(IntToStr(x));
  new(tmp);
  tmp^.Key := Edit1.Text;
  tmp^.Kind := TTokenType(ComboBox1.ItemIndex);
  FHashList[x].Add(tmp);
  ListBox1.Items.Add(Edit1.Text);
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  if SaveDialog1.Execute then
    SaveHLTable(SaveDialog1.FileName);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Button2Click(nil);
  Button3Click(nil);
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  i: integer;
begin
  Button6Click(nil);
  for i := 0 to 255 do
    FHashList[i].Free;
end;

procedure TForm1.ListBox1Click(Sender: TObject);
var x, i: Integer;
begin
  if ListBox1.ItemIndex>=0 then
  for x := 0 to 255 do
    for i := 0 to FHashList[x].Count - 1 do
      if PHashInfo(FHashList[x][i])^.Key = ListBox1.Items[ListBox1.ItemIndex] then
      begin
        Edit1.Text:=PHashInfo(FHashList[x][i])^.Key;
        ComboBox1.ItemIndex:=Ord(PHashInfo(FHashList[x][i])^.Kind);
      end;
end;

procedure TForm1.LoadHLTable(Path: string);
var
  fs: TFileStream;
  tmp: PHashInfo;
  i, x, n, a: integer;
begin
  Button6Click(nil);
  fs := TFileStream.Create(Path, fmOpenRead);
  try
    for x := 0 to 255 do
    begin
      fs.Read(n, SizeOf(n));
      for i := 0 to n - 1 do
      begin
        new(tmp);
        fs.Read(tmp^.Kind, SizeOf(tmp^.Kind));
        fs.Read(a, SizeOf(a));
        SetLength(tmp^.Key, a);
        fs.Read(tmp^.Key[1], a);
        FHashList[x].Add(tmp);
      end;
    end;
  finally
    fs.Free;
  end;
end;

procedure TForm1.SaveHlTable(Path: string);
var
  fs: TFileStream;
  tmp: THashInfo;
  i, x, a: integer;

begin
  fs := TFileStream.Create(Path, fmCreate);
  try
    for x := 0 to 255 do
    begin
      fs.Write(FHashList[x].Count, SizeOf(integer));
      for i := 0 to FHashList[x].Count - 1 do
      begin
        tmp := PHashInfo(FHashList[x][i])^;
        fs.Write(tmp.Kind, SizeOf(tmp.Kind));
        a := Length(tmp.Key);
        fs.Write(a, SizeOf(a));
        fs.Write(tmp.Key[1], a);
      end;
    end;
  finally
    fs.Free;
  end;

end;

end.
